#include <stdio.h>
#include <GL/glew.h>
#include "mesh.h"

Mesh::Mesh()
{
	vertices = 0;
	vcount = 0;
}

Mesh::Mesh(int vcount)
{
	vertices = 0;
	vcount = 0;

	set_vertex_count(vcount);
}

Mesh::~Mesh()
{
	delete [] vertices;
}

void Mesh::set_vertex_count(int vcount)
{
	delete [] vertices;

	if(vcount) {
		vertices = new Vertex[vcount];
	} else {
		vertices = 0;
	}
	this->vcount = vcount;
}

int Mesh::get_vertex_count() const
{
	return vcount;
}

Vertex *Mesh::get_vertex_array() const
{
	return vertices;
}

void Mesh::draw() const
{
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glVertexPointer(3, GL_FLOAT, sizeof(Vertex), &vertices->pos);
	glNormalPointer(GL_FLOAT, sizeof(Vertex), &vertices->normal);
	glTexCoordPointer(2, GL_FLOAT, sizeof(Vertex), vertices->texcoord);

	glDrawArrays(GL_TRIANGLES, 0, vcount);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
}


void Mesh::draw_normals() const
{
	glBegin(GL_LINES);
	glColor3f(0, 0, 1);

	for(int i=0; i<vcount; i++) {
		Vector3 v = vertices[i].pos;
		Vector3 n = vertices[i].normal;

		glVertex3f(v.x, v.y, v.z);
		glVertex3f(v.x + n.x, v.y + n.y, v.z + n.z);
	}

	glEnd();
}

void Mesh::draw_tangents() const
{
}
